//*****************************************************************************
//                               AppGSpiceUI.hpp                              *
//                              -----------------                             *
//  Description : This class does the command line processing.                *
//  Started     : 22/02/2005                                                  *
//  Last Update : 05/04/2005                                                  *
//  Copyright   : (C) 2005 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef CMDLINEPCR_HPP
#define CMDLINEPCR_HPP

// System includes

#include <iostream>

// wxWindows includes

#include <wx/wx.h>
#include <wx/cmdline.h>
#include <wx/filename.h>

// Application includes

#include "Version.hpp"
#include "AppGSpiceUI.hpp"
#include "FrmMain.hpp"
#include "FileTasks.hpp"

// Local constant declarations


//*****************************************************************************

class CmdLinePcr : wxCmdLineParser
{
  private:

    AppGSpiceUI * m_poApp;
    FrmMain *     m_poFrmMain;
    FileTasks *   m_poFileTsks;
    int           m_iArgC;
    wxChar **     m_ppsArgV;
    
    bool  bProcOption_a( int * piArg );  // Analysis page specifier
    bool  bProcOption_c( int * piArg );  // Specify a SPICE command file
    bool  bProcOption_g( int * piArg );  // Specify Guile procedure for import
    bool  bProcOption_h( int * piArg );  // Display usage and exit
    bool  bProcOption_l( int * piArg );  // Set max lines allowed in text ctrls
    bool  bProcOption_d( int * piArg );  // Set lines in text ctrl display area
    bool  bProcOption_t( int * piArg );  // Set temp. file management strategy
    bool  bProcOption_r( int * piArg );  // Specify a configuration file
    bool  bProcOption_s( int * piArg );  // Simulator engine specifier
    bool  bProcOption_v( int * piArg );  // Display app. version and exit
    bool  bProcFileName( int * piArg );  // Net list or schematic file name

    void  Usage( void );
    void  Version( void );

  public:

    CmdLinePcr( AppGSpiceUI * poApp );
    ~CmdLinePcr( );

    bool  bSetCmdLine( int iArgC, wxChar ** ppsArgV );
    bool  bProcArgs( void );
};

//*****************************************************************************

#endif // CMDLINEPCR_HPP
