//*****************************************************************************
//                                DlgSimrCfg.hpp                              *
//                               ----------------                             *
//  Description : This dialogue is used to enter values associated with the   *
//                electronic circuit simulators.                              *
//  Started     : 17/05/2004                                                  *
//  Last Update : 06/06/2005                                                  *
//  Copyright   : (C) 2004 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef DLGSIMRCFG_HPP
#define DLGSIMRCFG_HPP

// System Includes

#include <iostream>

// wxWidgets Includes

#include <wx/wx.h>
#include <wx/dialog.h>

// Application Includes


// Local Constant Declarations


//*****************************************************************************

class DlgSimrCfg : public wxDialog
{
  public:

    // Event identifiers
    enum eDlgItemID
    {
      ID_BTN_SIM_GNUCAP = 1,
      ID_BTN_SIM_NGSPICE,
      ID_BTN_HLP_GNUCAP,
      ID_BTN_HLP_NGSPICE,
      ID_BTN_OK,
      ID_BTN_CANCEL,

      ID_UNUSED,     // Assigned to controls for which events are not used

      ID_FST_SIMR = ID_BTN_SIM_GNUCAP,
      ID_LST_SIMR = ID_BTN_SIM_NGSPICE,

      ID_FST = ID_BTN_SIM_GNUCAP,
      ID_LST = ID_BTN_CANCEL
    };

  private:

    // Simulator binary names and locations
    wxTextCtrl  m_oTxtSimrCmd[ID_LST_SIMR];

    // Simulator help names and locations
    wxTextCtrl  m_oTxtHelpDoc[ID_LST_SIMR];

    // Button controls
    wxButton    m_oBtnSimrCmd[ID_LST_SIMR];
    wxButton    m_oBtnHelpDoc[ID_LST_SIMR];
    wxButton    m_oBtnOk;
    wxButton    m_oBtnCancel;

    // Temporary storage for original spin control values until Ok pressed
    wxString    m_osSimrCmd[ID_LST_SIMR];
    wxString    m_osHelpDoc[ID_LST_SIMR];

    // Object initialization functions
    void  Init    ( void );
    void  InitObjs( void );

  public:

    DlgSimrCfg( wxWindow * poWin );
    ~DlgSimrCfg( );

    void  Clear( void );

    bool  bSetSimrCmd( int iSimrID, const wxChar * psCmd );
    bool  bSetHelpDoc( int iSimrID, const wxChar * psDoc );

    const wxString & rosGetSimrCmd( int iSimrID );
    const wxString & rosGetHelpDoc( int iSimrID );

    // Event handlers
    void  OnBtnSimrCmd( wxCommandEvent & roEvtCmd );
    void  OnBtnHelpDoc( wxCommandEvent & roEvtCmd );
    void  OnBtnOk     ( wxCommandEvent & roEvtCmd );
    void  OnBtnCancel ( wxCommandEvent & roEvtCmd );

    // Leave this as the last line as private access is envoked by macro
    DECLARE_EVENT_TABLE( )
};

//*****************************************************************************

#endif // DLGSIMRCFG_HPP
