//*****************************************************************************
//                                FileTasks.hpp                               *
//                               ---------------                              *
//  Description : This is a helper class for FrmMain, it handles most of the  *
//                required file operations.                                   *
//  Started     : 28/05/2005                                                  *
//  Last Update : 18/06/2005                                                  *
//  Copyright   : (C) 2005 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef FILETASKS_HPP
#define FILETASKS_HPP

// System includes

#include <iostream>

// wxWindows includes

#include <wx/wx.h>
#include <wx/filename.h>

// Application includes

class FrmMain;

#include "Simulation.hpp"
#include "PrcGNetList.hpp"

// Local macro declarations

#if wxCHECK_VERSION(2,5,4)
#define  DELAY_MS(X)  wxMicroSleep(X)
#else
#define  DELAY_MS(X)  wxUsleep(X)
#endif

//*****************************************************************************

class FileTasks
{
  public:

    enum eTmpFileMgntType
    {
      eTFM_DELETE = 0,
      eTFM_PROMPT,
      eTFM_KEEP,

      eTFM_FST = eTFM_DELETE,
      eTFM_LST = eTFM_KEEP
    };

  private:

    FrmMain *         m_poFrmMain;    // Pointer to application main frame
    eTmpFileMgntType  m_eTmpFileMgnt; // Temporary file management strategy
    PrcGNetList       m_oPrcGNetList; // gNetList file conversion utility

    bool  bDlgOpen  ( void );
    bool  bDlgImport( void );

    bool  bLoadFile( Simulation & roSim );
    bool  bExecImport( void );

    void  DelTmpFiles( void );

  public:

    FileTasks( void );
    ~FileTasks( );

    bool  bClear( void );
    bool  bOkGNetList( void );

    void   SetAppFrm     ( FrmMain * poFrmMain ) { m_poFrmMain = poFrmMain; }
    bool  bSetSchems     ( const wxArrayString & roasFileNames );
    bool  bSetGuileProc  ( const wxString & rosProcName );
    bool  bSetNetList    ( const wxString & rosFileName );
    bool  bSetTmpFileMgnt( eTmpFileMgntType  eTmpFileMgnt );
    bool  bSetLogFile    ( const wxString & rosFileName )
                       { return( m_oPrcGNetList.bSetLogFile( rosFileName ) ); }

    const wxArrayString & roasGetSchems   ( void );
    const wxString      &  rosGetGuileProc( void );
    const wxString      &  rosGetNetList  ( void );

    bool  bOpen  ( Simulation & roSim );
    bool  bImport( Simulation & roSim );
    bool  bReload( Simulation & roSim );
    void   Close ( void );
    void   Exit  ( void );
};

//*****************************************************************************

#endif // FILETASKS_HPP
