//*****************************************************************************
//                                FrmMain.cpp                                 *
//                               -------------                                *
//  Started     : 18/08/2003                                                  *
//  Last Update : 06/07/2005                                                  *
//  Copyright   : (C) 2003 by MSWaters                                        *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#include "FrmMain.hpp"

// The application icon
#if defined( __WXGTK__ ) || defined( __WXMOTIF__ ) || defined( __WXMAC__ ) || \
    defined( __WXMGL__ ) || defined( __WXX11__ )
  #include "icon/gspiceui.xpm"
  #include "icon/fileopen.xpm"
  #include "icon/fileimport.xpm"
  #include "icon/filereload.xpm"
  #include "icon/fileclose.xpm"
  #include "icon/create.xpm"
  #include "icon/run.xpm"
  #include "icon/stop.xpm"
  #include "icon/plot.xpm"
#endif

//*****************************************************************************
// Implement an event table in which the events are routed to their respective
// handler functions in the class. If -1 is given as the ID, the given handler
// will be invoked for any event of the specified type.

BEGIN_EVENT_TABLE( FrmMain, wxFrame )

  EVT_MENU( ID_MNU_OPEN,    FrmMain::OnOpen    )
  EVT_MENU( ID_MNU_IMPORT,  FrmMain::OnImport  )
  EVT_MENU( ID_MNU_RELOAD,  FrmMain::OnReload  )
  EVT_MENU( ID_MNU_CLOSE,   FrmMain::OnClose   )
  EVT_MENU( ID_MNU_EXIT,    FrmMain::OnExit    )
  EVT_MENU( ID_MNU_CREATE,  FrmMain::OnCreate  )
  EVT_MENU( ID_MNU_RUN,     FrmMain::OnRun     )
  EVT_MENU( ID_MNU_STOP,    FrmMain::OnStop    )
  EVT_MENU( ID_MNU_PLOT,    FrmMain::OnPlot    )
  EVT_MENU( ID_MNU_SELGCAP, FrmMain::OnSelSimr )
  EVT_MENU( ID_MNU_SELNGSP, FrmMain::OnSelSimr )
  EVT_MENU( ID_MNU_PATHS,   FrmMain::OnPaths   )
  EVT_MENU( ID_MNU_TOOLTIP, FrmMain::OnToolTip )
  EVT_MENU( ID_MNU_CONTENT, FrmMain::OnContent )
  EVT_MENU( ID_MNU_HLPGCAP, FrmMain::OnGnuCap  )
  EVT_MENU( ID_MNU_HLPNGSP, FrmMain::OnNgSpice )
  EVT_MENU( ID_MNU_ABOUT,   FrmMain::OnAbout   )

  EVT_TOOL( ID_TBR_OPEN,    FrmMain::OnOpen    )
  EVT_TOOL( ID_TBR_IMPORT,  FrmMain::OnImport  )
  EVT_TOOL( ID_TBR_RELOAD,  FrmMain::OnReload  )
  EVT_TOOL( ID_TBR_CLOSE,   FrmMain::OnClose   )
  EVT_TOOL( ID_TBR_CREATE,  FrmMain::OnCreate  )
  EVT_TOOL( ID_TBR_RUN,     FrmMain::OnRun     )
  EVT_TOOL( ID_TBR_STOP,    FrmMain::OnStop    )
  EVT_TOOL( ID_TBR_PLOT,    FrmMain::OnPlot    )

  EVT_CHECKBOX( -1, FrmMain::OnParmCmd )

  EVT_NOTEBOOK_PAGE_CHANGED( -1, FrmMain::OnParmNbk )

  EVT_SIZE( FrmMain::OnSize )

END_EVENT_TABLE( )

//*****************************************************************************
// Constructor.
//
// Argument List:
//   poApp - A pointer to the class that created this object

FrmMain::FrmMain( const wxApp * poApp ) :
         wxFrame( (wxFrame *) NULL, -1, wxT(""), wxDefaultPosition, wxSize(FRMMAIN_WIDTH,500),
                  wxDEFAULT_FRAME_STYLE & ~(wxRESIZE_BORDER | wxRESIZE_BOX | wxMAXIMIZE_BOX) )
{
  // Set pointer to parent application
  m_poApp = poApp;

  // Set frame pointer in helper classes
  m_oFileTsks.SetAppFrm( this );
  m_oHelpTsks.SetAppFrm( this );

  // Initialize pointers
  m_poPrcSimr = NULL;
  m_poNbkSimr = NULL;
}

//*****************************************************************************
// Destructor.

FrmMain::~FrmMain( )
{
  if( m_poNbkSimr != NULL ) { delete m_poNbkSimr; m_poNbkSimr = NULL; }
  if( m_poPrcSimr != NULL ) { delete m_poPrcSimr; m_poPrcSimr = NULL; }
}

//*****************************************************************************
// Initialize the display objects.

void  FrmMain::Initialize( void )
{
  // Set the frame icon
  SetIcon( wxICON( gspiceui ) );

  InitToolBar ( );
  InitMenuBar ( );
  InitLstBoxs ( );
  InitNbkTCtls( );
  InitStatBar ( );
  InitToolTips( );
  InitLogFiles( );
  InitSimrEng ( );
}

//*****************************************************************************
// Initialize the menu bar.

void  FrmMain::InitMenuBar( void )
{
  // Create the menu bar
  wxMenuBar * poMenuBar  = new wxMenuBar;
  wxMenu    * poMenuFile = new wxMenu;
  wxMenu    * poMenuSimu = new wxMenu;
  wxMenu    * poMenuOpts = new wxMenu;
  wxMenu    * poMenuHelp = new wxMenu;

  poMenuFile->Append( ID_MNU_OPEN,   wxT(" &Open ...") );
  poMenuFile->Append( ID_MNU_IMPORT, wxT(" &Import ... ") );
  poMenuFile->Append( ID_MNU_RELOAD, wxT(" &Reload") );
  poMenuFile->Append( ID_MNU_CLOSE,  wxT(" &Close") );
  poMenuFile->AppendSeparator( );
  poMenuFile->Append( ID_MNU_EXIT,   wxT(" E&xit") );

  poMenuSimu->Append( ID_MNU_CREATE, wxT(" &Create    ") );
  poMenuSimu->Append( ID_MNU_RUN,    wxT(" &Run") );
  poMenuSimu->Append( ID_MNU_STOP,   wxT(" &Stop") );
  poMenuSimu->Append( ID_MNU_PLOT,   wxT(" &Plot") );

  poMenuOpts->AppendRadioItem( ID_MNU_SELGCAP, wxT("&GNU-Cap ") );
  poMenuOpts->AppendRadioItem( ID_MNU_SELNGSP, wxT("&NG-Spice ") );
  poMenuOpts->AppendSeparator( );
  poMenuOpts->Append         ( ID_MNU_PATHS,   wxT("&Paths ... ") );
  poMenuOpts->AppendSeparator( );
  poMenuOpts->AppendCheckItem( ID_MNU_TOOLTIP, wxT("&Tool Tips ") );

  poMenuHelp->Append( ID_MNU_CONTENT, wxT(" &Contents ... ") );
  poMenuHelp->AppendSeparator( );
  poMenuHelp->Append( ID_MNU_HLPGCAP, wxT(" &GNU-Cap ... ") );
  poMenuHelp->Append( ID_MNU_HLPNGSP, wxT(" &NG-Spice ... ") );
  poMenuHelp->AppendSeparator( );
  poMenuHelp->Append( ID_MNU_ABOUT,   wxT(" &About ... ") );

  poMenuBar->Append( poMenuFile, wxT(" &File ") );
  poMenuBar->Append( poMenuSimu, wxT(" &Simulate ") );
  poMenuBar->Append( poMenuOpts, wxT(" &Options ") );
  poMenuBar->Append( poMenuHelp, wxT(" &Help ") );

  SetMenuBar( poMenuBar );
}

//*****************************************************************************
// Initialize the tool bar.

void  FrmMain::InitToolBar( void )
{
  wxBitmap  * poBitmap[ 8 ];
  wxToolBar * poToolBar;

  // Create the tool bar
  poToolBar = CreateToolBar( wxHORIZONTAL | wxTB_FLAT );

  // Create the bitmaps for the tools
  poBitmap[ 0 ] = new wxBitmap( fileopen_xpm );
  poBitmap[ 1 ] = new wxBitmap( fileimport_xpm );
  poBitmap[ 2 ] = new wxBitmap( filereload_xpm );
  poBitmap[ 3 ] = new wxBitmap( fileclose_xpm );
  poBitmap[ 4 ] = new wxBitmap( create_xpm );
  poBitmap[ 5 ] = new wxBitmap( run_xpm );
  poBitmap[ 6 ] = new wxBitmap( stop_xpm );
  poBitmap[ 7 ] = new wxBitmap( plot_xpm );

  // Add the tools to the toolbar
  poToolBar->AddTool( ID_TBR_OPEN,   wxT(""), *(poBitmap[ 0 ]),
                      wxT("Open Netlist") );
  poToolBar->AddTool( ID_TBR_IMPORT, wxT(""), *(poBitmap[ 1 ]),
                      wxT("Import Schematic") );
  poToolBar->AddTool( ID_TBR_RELOAD, wxT(""), *(poBitmap[ 2 ]),
                      wxT("Reload Netlist") );
  poToolBar->AddTool( ID_TBR_CLOSE,  wxT(""), *(poBitmap[ 3 ]),
                      wxT("Close Netlist") );
  poToolBar->AddSeparator( );
  poToolBar->AddTool( ID_TBR_CREATE, wxT(""), *(poBitmap[ 4 ]),
                      wxT("Create Simulation") );
  poToolBar->AddTool( ID_TBR_RUN,    wxT(""), *(poBitmap[ 5 ]),
                      wxT("Run Simulation") );
  poToolBar->AddTool( ID_TBR_STOP,   wxT(""), *(poBitmap[ 6 ]),
                      wxT("Stop Simulation") );
  poToolBar->AddSeparator( );
  poToolBar->AddTool( ID_TBR_PLOT,   wxT(""), *(poBitmap[ 7 ]),
                      wxT("Plot Simulation Results") );

  // Realize the toolbar
  poToolBar->Realize( );

  // Can delete the bitmaps since they're reference counted
  for( int i1=0; i1<8; i1++ ) delete poBitmap[ i1 ];
}

//*****************************************************************************
// Initialize the test node and component list boxes.

void  FrmMain::InitLstBoxs( void )
{
  long  lStyle;

  // Create the node and component list labels
  lStyle = wxALIGN_CENTRE | wxST_NO_AUTORESIZE;
  m_oLblNodes.Create( this, ID_UNUSED, wxT("Nodes"),      wxPoint( 28, 5 ),
                      wxDefaultSize, lStyle );
  m_oLblCpnts.Create( this, ID_UNUSED, wxT("Components"), wxPoint( 108, 5 ),
                      wxDefaultSize, lStyle );

  // Create the node and component list boxes
  lStyle = wxLB_MULTIPLE | wxLB_EXTENDED | wxLB_ALWAYS_SB;
  m_oLbxNodes.Create( this, ID_LBX_NODES, wxPoint( 5, 27 ),
                      wxSize( 93, 248 ), 0, NULL, lStyle );
  m_oLbxCpnts.Create( this, ID_LBX_CPNTS, wxPoint( 102, 27 ),
                      wxSize( 93, 248 ), 0, NULL, lStyle );
}

//*****************************************************************************
// Initialize the console notebook.

void  FrmMain::InitNbkTCtls( void )
{
  int  i1;

  i1 = 40 + TextCtrl::iGetPixelHt( );

  m_oNbkTxtCtls.bCreate( this, ID_NBK_TXTCTLS, wxPoint( 5, 281 ),
                         wxSize( 730, i1 ) );
}

//*****************************************************************************
// Initialize the status bar.

void  FrmMain::InitStatBar( void )
{
  int    iWidths[2], ix, iy;
  float  f1;

  // Create the status bar
  CreateStatusBar( 2 );

  // Determine the status bar field widths
  GetSize( &ix, &iy );
  f1 = (float) (ix - 5);
  iWidths[ 0 ] = (int) (f1 * 0.70 + 0.5);
  iWidths[ 1 ] = (int) (f1 * 0.30 + 0.5);
  SetStatusWidths( 2, iWidths );

  // Set status bar text
  SetStatusText( wxT(" Welcome to GNU Spice GUI"), 0 );
  SetStatusText( wxT(" Simulation Engine : None"), 1 );
}

//*****************************************************************************
// Initialize the tool tips.

void  FrmMain::InitToolTips( void )
{
  // Define tool tips for each control
  m_oLblNodes.SetToolTip( wxT("Test Nodes") );      // ??? 21/02/2004 Doesn't work
  m_oLblCpnts.SetToolTip( wxT("Test Components") ); // ??? 21/02/2004 Doesn't work
  m_oLbxNodes.SetToolTip( wxT("Test Nodes") );
  m_oLbxCpnts.SetToolTip( wxT("Test Components") );

  // Set global tool tip attributes
  wxToolTip  oToolTip( wxT("") );
  oToolTip.Enable( FALSE );
  oToolTip.SetDelay( 700 );
}

//*****************************************************************************
// Set the electronic circuit simulator engine to use.
//
// Argument List:
//   eType - The simulator engine to use

void  FrmMain::InitSimrEng( Simulation::eSimrType eType )
{
  if( eType == Simulation::eSIMR_NONE ) return;

  // Delete any previous simulator objects
  if( m_poNbkSimr != NULL )
  { // Delete the old simulator notebook object
    if( m_poNbkSimr->eGetSimrType( ) == eType ) return; // Nothing to do
    delete m_poNbkSimr;
    m_poNbkSimr = NULL;
  }
  if( m_poPrcSimr != NULL )
  { // Delete the old simulator process object
    delete m_poPrcSimr;
    m_poPrcSimr = NULL;
  }

  // Create the new simulator objects
  switch( eType )
  {
    case Simulation::eSIMR_GNUCAP:
      // Create the notebook control which holds the various analysis pages
      m_poNbkSimr = new NbkGnuCap( this, ID_NBK_ANALYS, wxPoint( 200, 5 ),
                                   wxSize( 535, 270 ) );
      // Create the simulator process object
      m_poPrcSimr = new PrcGnuCap( );

      // Check the appropriate simulator item in the Options menu
      GetMenuBar( )->FindItem( FrmMain::ID_MNU_SELGCAP )->Check( );

      // Set the status bar message
      SetStatusText( wxT(" Simulation Engine : GNU-Cap"), 1 );
      break;

    case Simulation::eSIMR_NGSPICE:
      // Create the notebook control which holds the various analysis pages
      m_poNbkSimr = new NbkNgSpice( this, ID_NBK_ANALYS, wxPoint( 200, 5 ),
                                    wxSize( 535, 270 ) );
      // Create the simulator process object
      m_poPrcSimr = new PrcNgSpice( );

      // Check the appropriate simulator item in the Options menu
      GetMenuBar( )->FindItem( FrmMain::ID_MNU_SELNGSP )->Check( );

      // Set the status bar message
      SetStatusText( wxT(" Simulation Engine : NG-Spice"), 1 );
      break;

    default:
      break;
  }

  m_oSimn.bSetSimrType( eType );
  InitLogFiles( ); // Set the simulator process log file name
}

//*****************************************************************************
// Initialize the log file names in the process objects.

void  FrmMain::InitLogFiles( void )
{
  wxFileName  oFn1;
  wxString    os1;

  // Check that a schematic or netlist file has been defined
  oFn1 = m_oFileTsks.rosGetNetList( );
  if( ! oFn1.IsOk( )       ) return;
  if( ! oFn1.FileExists( ) ) return;

  // Get the path to the schematic or netlist file
  os1 = oFn1.GetPath( ) + wxT("/gspiceui.log");

  // Set the log file path for the gNetList process object
  m_oFileTsks.bSetLogFile( os1 );

  // Set the log file path for the simulation process object
  if( m_poPrcSimr != NULL ) m_poPrcSimr->bSetLogFile( os1 );

  // Set the log file path for the gWave process object
  m_oPrcGWave.bSetLogFile( os1 );
}

//*****************************************************************************
// Clear the object attributes.

bool  FrmMain::bClear( void )
{
  bool  bRtn=TRUE;

  // Clear all previously selected test points
  m_oLbxNodes.Clear( );
  m_oLbxCpnts.Clear( );

  // Clear the simulation object and the analysis panels
  if( m_poNbkSimr != NULL )
    if( ! m_poNbkSimr->bClear( ) ) bRtn = FALSE;

  // Clear the text controls
  if( ! m_oNbkTxtCtls.bClear( ) )  bRtn = FALSE;
  m_oNbkTxtCtls.Initialize( );

  // Clear all GNetList process object attributes
  if( ! m_oFileTsks.bClear( ) )    bRtn = FALSE;

  // Terminate any viewer processes
  if( ! m_oPrcGWave.bKill( ) )     bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
// Load information from the Simulation object.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  FrmMain::bLoad( void )
{
  TextCtrl * poTxtCtl;
  uint    ui1;
  size_t  szt1;

  // Parse the simulator commands
  InitSimrEng( m_oSimn.eGetSimrType( ) );
  m_poPrcSimr->bParseSim( m_oSimn );

  // Load the nodes into the list box
  const wxArrayString & roasNodeLbls = m_oSimn.roasGetNodeLbls( );
  for( ui1=0; ui1<roasNodeLbls.GetCount( ); ui1++ )
    m_oLbxNodes.Append( roasNodeLbls.Item( ui1 ) );
  const wxArrayString & roasTstNodes = m_oSimn.roasGetTstNodes( );
  for( ui1=0; ui1<roasTstNodes.GetCount( ); ui1++ )
    m_oLbxNodes.SetStringSelection( roasTstNodes.Item( ui1 ) );

  // Load the components into the list box
  const wxArrayString & roasCpntLbls = m_oSimn.roasGetCpntLbls( );
  for( ui1=0; ui1<roasCpntLbls.GetCount( ); ui1++ )
    m_oLbxCpnts.Append( roasCpntLbls.Item( ui1 ) );
  const wxArrayString & roasTstCpnts = m_oSimn.roasGetTstCpnts( );
  for( ui1=0; ui1<roasTstCpnts.GetCount( ); ui1++ )
    m_oLbxCpnts.SetStringSelection( roasTstCpnts.Item( ui1 ) );

  // Load the analysis notebook with circuit description information
  if( m_poNbkSimr != NULL ) m_poNbkSimr->bLoad( m_oSimn );

  // Load the raw circuit description into the text control
  m_oNbkTxtCtls.bClear( NbkTxtCtls::ePAGE_NETLIST );
  poTxtCtl = m_oNbkTxtCtls.poGetPage( NbkTxtCtls::ePAGE_NETLIST );
  poTxtCtl->Clear( );
  for( szt1=0; szt1<m_oSimn.GetCount( ); szt1++ )
    poTxtCtl->bAppendLine( m_oSimn.Item( szt1 ) );
  poTxtCtl->SetInsertionPoint( 0 ); // Go to top of the netlist text control

  return( TRUE );
}

//*****************************************************************************
// Reload information from the Simulation object.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  FrmMain::bReLoad( void ) // ???
{
//  wxArrayString  oasNodes;
//  wxString  os1;
//  size_t  szt1;
//  int  i1;

  // Clear the list of nodes and components
  m_oLbxNodes.Clear( );
  m_oLbxCpnts.Clear( );

  bLoad( ); // Load the Simulation object

  // Check whether any off the test nodes or test components no longer exist  
  



  // Get the nodes that have been selected
//  for( i1=0; i1<m_oLbxNodes.GetCount( ); i1++ )
//  {
//    if( m_oLbxNodes.IsSelected( i1 ) )
//    {
//      os1 = m_oLbxNodes.GetString( i1 );
//      oasNodes.Add( os1 );
//    }
//  }

  // Select nodes that where previously selected
//  if( ! oasNodes.IsEmpty( ) )
//  {
//    for( szt1=0; szt1<oasNodes.GetCount( ); szt1++ )
//    {
//      if( (i1=m_oLbxNodes.FindString( oasNodes.Item( szt1 ) )) != wxNOT_FOUND )
//        m_oLbxNodes.Select( i1 );
//    }
//  }

  return( FALSE );
}
  
//*****************************************************************************
// Save information to the Simulation object.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  FrmMain::bSave( void )
{
  wxArrayInt  oaiSelNodes, oaiSelCpnts;
  size_t      szt1;
  int         iSelNo;
  wxString    osSel, osErrMsg;

  // Clear all previously selected test points
  m_oSimn.ClrTstNodes( );
  m_oSimn.ClrTstCpnts( );

  // Get an array of indexes to the selected test points
  m_oLbxNodes.GetSelections( oaiSelNodes );
  m_oLbxCpnts.GetSelections( oaiSelCpnts );

  // Have any test points (ie. nodes or components) been selected?
  if( oaiSelNodes.GetCount( )>0 || oaiSelCpnts.GetCount( )>0 )
  {
    // Load the test nodes into the simulation object
    for( szt1=0; szt1<oaiSelNodes.GetCount( ); szt1++ )
    {
      iSelNo = oaiSelNodes.Item( szt1 );
      osSel = m_oLbxNodes.GetString( iSelNo );
      m_oSimn.bAddTstNode( osSel );
    }

    // Load the test components into the simulation object
    for( szt1=0; szt1<oaiSelCpnts.GetCount( ); szt1++ )
    {
      iSelNo = oaiSelCpnts.Item( szt1 );
      osSel = m_oLbxCpnts.GetString( iSelNo );
      m_oSimn.bAddTstCpnt( (const wxString &) osSel );
    }

    // Setup the simulation parameters
    if( m_poNbkSimr->bSave( m_oSimn ) )
    {
      // Create the simulation file
      if( ! m_poPrcSimr->bMakeSim( m_oSimn ) )
        osErrMsg = wxT("Couldn't create simulation.");
    }
    else osErrMsg = m_poNbkSimr->psGetError( );
  }
  else osErrMsg = wxT("No nodes or components have been selected.");

  if( ! osErrMsg.IsEmpty( ) )
  {
    ::wxSetCursor( wxNullCursor );
    DlgErrMsg( wxT("Create Simulation Error"), osErrMsg );
    return( FALSE );
  }

  return( TRUE );
}

//*****************************************************************************
// Display a dialog box containing an error message.
//
// Argument List:
//   rosTitle - The dialogue box title
//   rosMsg   - The error message

void  FrmMain::DlgErrMsg( const wxString & rosTitle, const wxString & rosMsg )
{
  wxString  os1;

  if( IsShown( ) )
  {
    os1 << rosTitle << wxT(" : ") << rosMsg;
    SetStatusText( os1 );
    wxMessageDialog  oMsgDlg( this, rosMsg, rosTitle, wxOK | wxICON_ERROR );
    oMsgDlg.ShowModal( );
  }
}

//*****************************************************************************
// Check that the electronic circuit simulator engine is present and accounted
// for, if not display an error message.
//
// Argument List:
//   poPrcSimr - The utility object to be tested
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  FrmMain::bOkSimr( PrcSimrBase * poPrcSimr )
{
  wxString  osErrMsg;

  // Check that gnetlist exists and is accessible
  if( ! poPrcSimr->bBinExists( ) )
  {
    osErrMsg << wxT("\nCan't find ") << poPrcSimr->roGetBinary( ).GetFullName( )
             << wxT(" which is required to run electronic simulation.\n")
             << wxT("There is no path to it or it has not been installed.\n\n");
    DlgErrMsg( wxT("Configuration Fault"), osErrMsg );
    return( FALSE );
  }

  return( TRUE );
}

//*****************************************************************************
// Check that the results plotter utility is present and accounted for, if not
// display an error message.
//
// Argument List:
//   poPrcGWave - The utility object to be tested
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  FrmMain::bOkGWave( PrcGWave * poPrcGWave )
{
  wxString  os1;

  // Check that the plotter utility exists and is accessible
  if( ! poPrcGWave->bBinExists( ) )
  {
    os1 << wxT("\nCan't find ") << poPrcGWave->roGetBinary( ).GetFullName( )
        << wxT(" which is required to plot simulation results.\n")
        << wxT("There is no path to it or it has not been installed.\n\n");
    DlgErrMsg( wxT("Configuration Fault"), os1 );
    return( FALSE );
  }

  return( TRUE );
}

//*****************************************************************************
// Set the frame title.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  FrmMain::bSetTitle( void )
{
  wxString  os1;
  size_t    szt1;

  os1 << APP_NAME << wxT(",  ") << APP_VERSION;

  const wxArrayString & roas1 = m_oFileTsks.roasGetSchems( );
  if( roas1.GetCount( ) > 0 )
  {
    os1 << wxT("  -  ");
    for( szt1=0; szt1<roas1.GetCount( ); szt1++ )
    {
      if( szt1 > 0 ) os1 << wxT(", ");
      os1 << roas1.Item( szt1 );
    }
  }
  else if( ! m_oFileTsks.rosGetNetList( ).IsEmpty( ) )
    os1 << wxT("  -  ") << m_oFileTsks.rosGetNetList( );

  wxFrame::SetTitle( os1 );

  return( TRUE );
}

//*****************************************************************************
// Set the schematic file name/s.
//
// Argument List:
//   roasFileName - A string array containing the full path and file name/s
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  FrmMain::bSetSchems( const wxArrayString & roasFileName )
{
  if( ! m_oFileTsks.bSetSchems( roasFileName ) ) return( FALSE );

  bSetTitle( );    // Set frame title text
  InitLogFiles( ); // Initialize the process log file

  return( TRUE );
}

//*****************************************************************************
// Set the net list file name.
//
// Argument List:
//   rosFileName - A string containing the full path and file name
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  FrmMain::bSetNetList( const wxString & rosFileName )
{
  if( ! m_oFileTsks.bSetNetList( rosFileName ) ) return( FALSE );

  bSetTitle( );    // Set frame title text
  InitLogFiles( ); // Initialize the process log file

  return( TRUE );
}

//*****************************************************************************
//                                                                            *
//                               Event Handlers                               *
//                                                                            *
//*****************************************************************************
// Open a circuit description file.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnOpen( wxCommandEvent & roEvtCmd )
{
  // Get a net list file name from the user and attempt to open it
  if( m_oFileTsks.bOpen( m_oSimn ) )
  {
    bClear( ); // Clear object attributes

    // Attempt to load the Simulation information
    if( bLoad( ) )
         SetStatusText( wxT(" Net list file opened successfully") );
    else SetStatusText( wxT(" Error/s encountered opening a net list file") );

    m_oNbkTxtCtls.bSetPage( NbkTxtCtls::ePAGE_NETLIST );
    m_oNbkTxtCtls.bSetPosn( 0 ); // Go to the top of the netlist text control
    DELAY_MS( 100 );             // Allow time for the GUI to be updated
  }
}

//*****************************************************************************
// Import a schematic file using gnetlist to convert the schematic to a circuit
// description.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnImport( wxCommandEvent & roEvtCmd )
{
  // Get schematic file name/s from the user and attempt to import them
  if( m_oFileTsks.bImport( m_oSimn ) )
  {
    bClear( ); // Clear object attributes

    // Attempt to load the Simulation information
    if( bLoad( ) )
         SetStatusText( wxT(" Schematic file/s imported successfully") );
    else SetStatusText( wxT(" Error/s encountered importing schematic file/s") );

    m_oNbkTxtCtls.bSetPage( NbkTxtCtls::ePAGE_NETLIST );
    m_oNbkTxtCtls.bSetPosn( 0 ); // Go to the top of the netlist text control
    DELAY_MS( 100 );             // Allow time for the GUI to be updated
  }
}

//*****************************************************************************
// Reload the schematic or net list file.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnReload( wxCommandEvent & roEvtCmd )
{
  // Get schematic file name/s from the user and attempt to import them
  if( m_oFileTsks.bReload( m_oSimn ) )
  {
    // Clear the list of nodes and components
    m_oLbxNodes.Clear( );
    m_oLbxCpnts.Clear( );

    // Attempt to load the Simulation information
    if( bLoad( ) )
         SetStatusText( wxT(" File/s reloaded successfully") );
    else SetStatusText( wxT(" Error/s encountered reloading file/s") );

    m_oNbkTxtCtls.bSetPage( NbkTxtCtls::ePAGE_NETLIST );
    m_oNbkTxtCtls.bSetPosn( 0 ); // Go to the top of the netlist text control
    DELAY_MS( 100 );             // Allow time for the GUI to be updated
  }
}

//*****************************************************************************
// Close the circuit description file.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnClose( wxCommandEvent & roEvtCmd )
{
  m_oFileTsks.Close( );                         // Delete temporary files
  bClear( );                                    // Clear the object attributes
  bSetTitle( );                                 // Set the frame title
  m_poNbkSimr->bSetPage( Simulation::eANA_OP ); // Display first analysis panel
}

//*****************************************************************************
// Close the application.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnExit( wxCommandEvent & roEvtCmd )
{
//wxLogDebug( "FrmMain::OnExit( )\n" );

  m_oFileTsks.Exit( ); // Delete temporary files
  Close( TRUE );       // Close the window
}

//*****************************************************************************
// Create the simulation.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnCreate( wxCommandEvent & roEvtCmd )
{
  TextCtrl * poTxtCtl;
  wxString  osErrMsg;

  // Change the cursor to the wait symbol
  ::wxSetCursor( wxCursor( wxCURSOR_WAIT ) );

  if( ! m_oSimn.IsEmpty( ) )
  {
    m_oNbkTxtCtls.bSetPage( NbkTxtCtls::ePAGE_CONSOLE );
    DELAY_MS( 100 ); // Allow time for the GUI to be updated

    if( bSave( ) )
    {
      // Print the simulation to the simulation text control
      m_oNbkTxtCtls.bSetPage( NbkTxtCtls::ePAGE_SIMULTN );
      poTxtCtl = m_oNbkTxtCtls.poGetPage( NbkTxtCtls::ePAGE_SIMULTN );
      poTxtCtl->bLoadFile( m_oSimn.roGetSaveFile( ).GetFullPath( ) );
      poTxtCtl->SetEditable( TRUE );
      DELAY_MS( 100 ); // Allow time for the GUI to be updated
    }
    else
      SetStatusText( wxT(" Error/s encountered creating simulation") );
  }
  else osErrMsg = wxT("There is no net list loaded.");

  // Change the cursor to the default
  ::wxSetCursor( wxNullCursor );

  if( ! osErrMsg.IsEmpty( ) )
  { // Display an error message dialogue
    SetStatusText( wxT(" Error/s encountered creating simulation") );
    DlgErrMsg( wxT("Create Simulation Error"), osErrMsg );
  }
  else SetStatusText( wxT(" Simulation created successfully") );
}

//*****************************************************************************
// Run the simulation.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnRun( wxCommandEvent & roEvtCmd )
{
  TextCtrl * poTxtCtl;
  wxString  osErrMsg, os1;

  // Change the cursor to the wait symbol
  ::wxSetCursor( wxCursor( wxCURSOR_WAIT ) );

  // Load the simulation text control
  if( ! m_oSimn.IsEmpty( ) )
  {
    m_oNbkTxtCtls.bSetPage( NbkTxtCtls::ePAGE_SIMULTN );
    DELAY_MS( 100 ); // Allow time for the GUI to be updated

    poTxtCtl = m_oNbkTxtCtls.poGetPage( NbkTxtCtls::ePAGE_SIMULTN );
    if( poTxtCtl->IsModified( ) )
    { // Save the simulation in the text control to file
      poTxtCtl->SaveFile( m_oSimn.roGetSaveFile( ).GetFullPath( ) );
    }
    else if( bSave( ) )
    { // Print the simulation to the simulation text control
      poTxtCtl->bLoadFile( m_oSimn.roGetSaveFile( ).GetFullPath( ) );
      // poTxcSimn is saved immediately after loading to clear IsModified flag
      poTxtCtl->SaveFile( m_oSimn.roGetSaveFile( ).GetFullPath( ) );
      poTxtCtl->SetEditable( TRUE );
      DELAY_MS( 100 ); // Allow time for the GUI to be updated
    }
    else goto OnRunError;

    // Run the simulation
    SetStatusText( wxT(" Running the simulation ...") );
    m_oNbkTxtCtls.bSetPage( NbkTxtCtls::ePAGE_CONSOLE );
    DELAY_MS( 100 ); // Allow time for the GUI to be updated

    if( ! m_poPrcSimr->bExec( ) ) osErrMsg = m_poPrcSimr->psGetError( );
    // Load the simulation output into the console text controls
    poTxtCtl = m_oNbkTxtCtls.poGetPage( NbkTxtCtls::ePAGE_CONSOLE );
    m_poPrcSimr->Print( *poTxtCtl );  // Print the console output
    DELAY_MS( 100 ); // Allow time for the GUI to be updated
    // Return if problems were encountered while running the simulation
    if( ! osErrMsg.IsEmpty( ) ) goto OnRunError;

    // Load the simulation output into the results text controls
    poTxtCtl = m_oNbkTxtCtls.poGetPage( NbkTxtCtls::ePAGE_RESULTS );
    os1 = m_poPrcSimr->roGetResultsFile( ).GetFullPath( );
    if( poTxtCtl->bLoadFile( os1 ) )  // Print results
    {
      DELAY_MS( 100 ); // Allow time for the GUI to be updated
      m_oNbkTxtCtls.bSetPage( NbkTxtCtls::ePAGE_RESULTS );
    }
    else osErrMsg = wxT("Couldn't load the results file : ") + os1;

    // Set the results file in the simulation results viewer object
    m_oPrcGWave.bSetResults( os1 );
  }
  else osErrMsg = wxT("There is no net list loaded.");

OnRunError:

  // Change the cursor to the default
  ::wxSetCursor( wxNullCursor );

  if( !osErrMsg.IsEmpty( ) || m_poPrcSimr->bIsError( ) )
  { // Display an error message dialogue
    SetStatusText( wxT(" Error/s encountered running simulation") );
    DlgErrMsg( wxT( "Run Simulation Error" ), osErrMsg );
  }
  else SetStatusText( wxT(" Simulation ran successfully") );
}

//*****************************************************************************
// Stop the simulation.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnStop( wxCommandEvent & roEvtCmd )
{
  m_poPrcSimr->bKill( );

  SetStatusText( wxT(" Simulation aborted") );
}

//*****************************************************************************
// Plot the simulation results.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnPlot( wxCommandEvent & roEvtCmd )
{
  TextCtrl * poTxtCtl;
  wxString  os1;

  // Execute the viewer process
  if( ! m_oPrcGWave.bIsExec( ) )
  {
    if( m_oPrcGWave.bExec( ) )
    {
      // Load the plotter output into the console text control
      poTxtCtl = m_oNbkTxtCtls.poGetPage( NbkTxtCtls::ePAGE_CONSOLE );
      m_oPrcGWave.Print( *poTxtCtl ); // Print the results
      os1 = wxT("");
    }
    else os1 = m_oPrcGWave.psGetError( );
  }
  else os1 = wxT("A GWave process already running.");

  if( ! os1.IsEmpty( ) ) DlgErrMsg( wxT( "Plot Results Error" ), os1 );
}

//*****************************************************************************
// Select which simulator to use.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnSelSimr( wxCommandEvent & roEvtCmd )
{
  if( ! roEvtCmd.IsChecked( ) ) return;  

  switch( roEvtCmd.GetId( ) )
  {
    case ID_MNU_SELGCAP:
      InitSimrEng( Simulation::eSIMR_GNUCAP );
      break;

    case ID_MNU_SELNGSP:
      InitSimrEng( Simulation::eSIMR_NGSPICE );
      break;

    default:
      return;
  }

  if( m_poNbkSimr != NULL ) m_poNbkSimr->bLoad( m_oSimn );
}

//*****************************************************************************
// Set/display the paths names to the various utilities used bu GSpiceUI.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnPaths( wxCommandEvent & roEvtCmd )
{
  wxMessageBox( wxT("\n     Not Yet Implemented.     \n "),
                wxT("Paths"), wxOK | wxCENTRE, this );
}

//*****************************************************************************
// Eable / disable tool tips.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnToolTip( wxCommandEvent & roEvtCmd )
{
  static  bool  bState=FALSE;
  wxToolTip  oToolTip( wxT("") );

  bState = ! bState;

  oToolTip.Enable( bState );
}

//*****************************************************************************
// Display the GSpiceUI help contents.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnContent( wxCommandEvent & roEvtCmd )
{
  m_oHelpTsks.Content( );
}

//*****************************************************************************
// Display the GNU-Cap manual.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnGnuCap( wxCommandEvent & roEvtCmd )
{
  m_oHelpTsks.GnuCap( );
}

//*****************************************************************************
// Display the NG-Spice manual.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnNgSpice( wxCommandEvent & roEvtCmd )
{
  m_oHelpTsks.NgSpice( );
}

//*****************************************************************************
// Display about message dialog.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnAbout( wxCommandEvent & roEvtCmd )
{
  m_oHelpTsks.About( );
}

//*****************************************************************************
// Used to set the state of the node name list box control. The node name list
// box is only enabled when the voltage parameter check box is checked.

void  FrmMain::OnParm( void )
{
  if( m_poNbkSimr == NULL ) return;

  PnlAnaBase * poPnlAna = NULL;
  poPnlAna = (PnlAnaBase *) m_poNbkSimr->GetPage( m_poNbkSimr->GetSelection( ) );
  if( poPnlAna == NULL )    return;

  if( poPnlAna->m_oCbxVoltage.IsChecked( ) ) m_oLbxNodes.Enable( );
  else                                       m_oLbxNodes.Disable( );
}

//*****************************************************************************
// Event handler for when a new analysis panel is chosen.
//
// Argument List:
//   roEvtNbk - The event to be processed

void  FrmMain::OnParmNbk( wxNotebookEvent & roEvtNbk )
{
  OnParm( );
}

//*****************************************************************************
// Event handler for when a parameter check box changes state.
//
// Argument List:
//   roEvtCmd - The event to be processed

void  FrmMain::OnParmCmd( wxCommandEvent & roEvtCmd )
{
  OnParm( );
}

//*****************************************************************************
