//*****************************************************************************
//                                NbkSimrBase.hpp                             *
//                               -----------------                            *
//  Description : Base class for all simulator notebooks which contains the   *
//                individual analysis panels.                                 *
//  Started     : 05/05/2004                                                  *
//  Last Update : 06/06/2005                                                  *
//  Copyright   : (C) 2004 by MSWaters                                        *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef NBKSIMRBASE_HPP
#define NBKSIMRBASE_HPP

// System Includes

#include <iostream>

// wxWindows Includes

#include <wx/wx.h>
#include <wx/notebook.h>

// Application Includes

#include "NetList.hpp"
#include "Simulation.hpp"
#include "PnlAnaBase.hpp"

// Local Constant Declarations


//*****************************************************************************

class NbkSimrBase : public wxNotebook
{
  private:

    Simulation::eSimrType  m_eSimrType;

  protected:

    bool  bSetSimrType( Simulation::eSimrType eType );

  public:

    NbkSimrBase( wxWindow * poParent, wxWindowID oWinID,
                 const wxPoint & roPosn, const wxSize & roSize );
    NbkSimrBase( void );
    ~NbkSimrBase( );

    virtual  bool  bClear( void ) = 0;

    virtual  bool  bLoad( Simulation & roSim ) = 0;
    virtual  bool  bSave( Simulation & roSim ) = 0;

    virtual  bool  bSetPage( Simulation::eAnaType eType ) = 0;
    virtual  bool  bSetPage( const wxChar * psType ) = 0;

    Simulation::eSimrType  eGetSimrType( void ) { return( m_eSimrType ); }

    const wxChar * psGetError( void );

    void  OnSelChange( wxNotebookEvent & roEvent ); // ???

    // Leave this as the last line as private access is envoked by macro
    DECLARE_EVENT_TABLE( )
};

//*****************************************************************************

#endif // NBKSIMRBASE_HPP
