//*****************************************************************************
//                                PnlGnuCapFO.hpp                             *
//                               -----------------                            *
//  Description : This class derives from the Analysis Panel base class and   *
//                provides a GUI for a user to configure a Fourier Analysis   *
//                for GNU-Cap electronic circuit simulator.                   *
//  Started     : 26/04/2004                                                  *
//  Last Update : 01/07/2005                                                  *
//  Copyright   : (C) 2004 by MSWaters                                        *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef PNLGNUCAPFO_HPP
#define PNLGNUCAPFO_HPP

// System Includes

#include <cfloat>

// wxWidgets Library Includes


// Application Includes

#include "PnlAnaBase.hpp"
#include "DlgGenCfg.hpp"

// Local Constant Declarations


//*****************************************************************************

class PnlGnuCapFO : public PnlAnaBase
{
  private:

    // Generator configuration button
    wxChoice   m_oChoGenCpnt;
    wxButton   m_oBtnSetup;

    // Generator configuration object
    DlgGenCfg  m_oDlgGenCfg;

    void  Create( void );
    void  InitGen( void );

  public:

    PnlGnuCapFO( wxWindow * poWin );
    ~PnlGnuCapFO( );

    bool  bClear( void );

    bool  bLoad( Simulation & roSim );
    bool  bSave( Simulation & roSim );

    // Event handlers
    void  OnBtnSetup( wxCommandEvent & roEvtCmd );

    // In order to be able to react to a menu command, it must be given a
    // unique identifier such as a const or an enum.
    enum ePnlItemID
    {
      ID_CHO_GENCPNT = PnlAnaBase::ID_LST+1,
      ID_BTN_SETUP,

      ID_UNUSED,      // Assigned to controls for which events are not used

      ID_FST = ID_CHO_GENCPNT,
      ID_LST = ID_BTN_SETUP
    };

    // Leave this as the last line as private access is envoked by macro
    DECLARE_EVENT_TABLE( )
};

//*****************************************************************************

#endif // PNLGNUCAPFO_HPP
