//*****************************************************************************
//                                GNetList.hpp                                *
//                               --------------                               *
//  Description : Provides a programatic interface to the utility gnetlist    *
//                which is used to generate net list file from various        *
//                schematic editor formats.                                   *
//  Started     : 29/01/2004                                                  *
//  Last Update : 18/06/2005                                                  *
//  Copyright   : (C) 2004 by MSWaters                                        *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef PRCGNETLIST_HPP
#define PRCGNETLIST_HPP

// System Includes

#include <iostream>

// wxWindows Includes

#include <wx/wx.h>
#include <wx/filename.h>
#include <wx/dynarray.h>
#include <wx/regex.h>

WX_DECLARE_OBJARRAY( wxFileName, ArrayFileName );

// Application Includes

#include "PrcBase.hpp"

// Local Constant Declarations


//*****************************************************************************

class PrcGNetList : public PrcBase
{
  private:

    ArrayFileName  m_oaFnmSchems;   // The full names of the schematic file/s
    wxFileName     m_oFnmNetList;   // The full name  of the list file
    size_t         m_sztGuileProc;  // The Guile procedure specifier
    wxArrayString  m_oasGuileProcs; // Array of all possible Guile procedures

    void  InitGuileProcs( void );

  public:

    PrcGNetList( void );
    ~PrcGNetList( );

    bool  bClear( void );

    bool  bSetGNetList ( const wxString & rosFileName );
    bool  bSetSchem    ( const wxString & rosFileName );
    bool  bSetSchems   ( const wxArrayString & roasFileNames );
    bool  bSetNetList  ( const wxString & rosFileName=wxT("") );
    bool  bSetGuileProc( const wxString & rosGuileProc=wxT("") );

    const wxFileName    & roGetGNetList   ( void ) { return( roGetBinary( ) ); }
    const ArrayFileName & roGetSchems     ( void ) { return( m_oaFnmSchems ); }
    const wxFileName    & roGetSchem      ( size_t sztIndex=0 );
    const wxFileName    & roGetNetList    ( void ) { return( m_oFnmNetList ); }
    const wxArrayString & roaGetGuileProcs( void ) { return( m_oasGuileProcs ); }
    const wxString      & rosGetGuileProc ( size_t sztIndex=0 );
    size_t  sztGetGuileProcCnt( void ) { return( m_oasGuileProcs.GetCount( ) ); }
    size_t  sztGetSchemCnt    ( void ) { return( m_oaFnmSchems.GetCount( ) ); }

    bool  bExec( void );
};

//*****************************************************************************

#endif // PRCGNETLIST_HPP
