//*****************************************************************************
//                               PrcGnucap.hpp                                *
//                              ---------------                               *
//  Description : This class provides the interface to the GNU-Cap electronic *
//                circuit simulator.                                          *
//  Started     : 01/09/2003                                                  *
//  Last Update : 22/03/2005                                                  *
//  Copyright   : (C) 2003 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************
// The following is a list of files which may be created by GSpiceUI when     *
// using this class:                                                          *
//                                                                            *
// Source file:                                                               *
//   <Path>/test-amp1.sch          : Schematic file   OR                      *
//   <Path>/test-amp1.ckt          : Net list                                 *
//                                                                            *
// Files created by GSpiceUI:                                                 *
//   <Path>/test-amp1.gspiceui.ckt : Net list file with simulation commands   *
//   <Path>/gspiceui.log           : Temp. file containing raw process output *
//   <Path>/test-amp1.gnucap.op    : Results file - operating point analysis  *
//   <Path>/test-amp1.gnucap.dc    : Results file - DC analysis               *
//   <Path>/test-amp1.gnucap.ac    : Results file - AC analysis               *
//   <Path>/test-amp1.gnucap.tr    : Results file - transient analysis        *
//   <Path>/test-amp1.gnucap.fo    : Results file - fourier analysis          *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef PRCGNUCAP_HPP
#define PRCGNUCAP_HPP

// System Includes


// wxWindows Includes


// Application Includes

#include "PrcSimrBase.hpp"
#include "ConvertType.hpp"
#include "PnlGnuCapOP.hpp"
#include "PnlGnuCapDC.hpp"
#include "PnlGnuCapAC.hpp"
#include "PnlGnuCapTR.hpp"
#include "PnlGnuCapFO.hpp"

// Local Constant Declarations


//*****************************************************************************

class PrcGnuCap : public PrcSimrBase
{
  private:

    // Operator (X axis variable)
    wxString  m_osInput;

    // Functions to parse simulator commands
    bool  bParseCmdOPT( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdGEN( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdOP ( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdDC ( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdAC ( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdTR ( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdFO ( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdPR ( wxString & rosCmd, Simulation & roSim );

    // Functions to create the simulator commands
    void  MakeCmdOP( Simulation & roSim );
    void  MakeCmdDC( Simulation & roSim );
    void  MakeCmdAC( Simulation & roSim );
    void  MakeCmdTR( Simulation & roSim );
    void  MakeCmdFO( Simulation & roSim );
    bool  bMakeCmds( Simulation & roSim );

    // Create arguments for the PRINT command
    wxString & rosMakeArgsPR( Simulation & roSim, wxString & rosPrefix );

    // Functions to check / format the circuit description lines
    bool  bFmtCpnts( wxArrayString & roasCpnts );

    // Format the results file
    bool  bFmtResultsFile( void );
    bool  bFmtResultsHead( wxString & rosLine );
    bool  bFmtResultsLine( wxString & rosLine );

  public:

    PrcGnuCap( void );
    ~PrcGnuCap( );

    virtual  bool  bParseSim( Simulation & roSim );

    virtual  bool  bMakeSim( Simulation & roSim );

    virtual  bool  bExec( void );
};

//*****************************************************************************

#endif // PRCGNUCAP_HPP
