//*****************************************************************************
//                               PrcNgSpice.hpp                               *
//                              ----------------                              *
//  Description : This class provides the interface to the NG-Spice circuit   *
//                simulator.                                                  *
//  Started     : 07/05/2004                                                  *
//  Last Update : 22/03/2005                                                  *
//  Copyright   : (C) 2004 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************
// The following is a list of files which may be created by GSpiceUI when     *
// using this class:                                                          *
//                                                                            *
// Source file:                                                               *
//   <Path>/test-amp1.sch          : Schematic file   OR                      *
//   <Path>/test-amp1.ckt          : Net list                                 *
//                                                                            *
// Files created by GSpiceUI:                                                 *
//   <Path>/test-amp1.gspiceui.ckt : Net list file with simulation commands   *
//   <Path>/gspiceui.log           : Temp. file containing raw process output *
//   <Path>/test-amp1.ngspice.dc   : Results file - DC analysis               *
//   <Path>/test-amp1.ngspice.ac   : Results file - AC analysis               *
//   <Path>/test-amp1.ngspice.di   : Results file - distortion analysis       *
//   <Path>/test-amp1.ngspice.no   : Results file - noise analysis            *
//   <Path>/test-amp1.ngspice.pz   : Results file - pole-zero analysis        *
//   <Path>/test-amp1.ngspice.se   : Results file - sensitivity analysis      *
//   <Path>/test-amp1.ngspice.tr   : Results file - transient analysis        *
//   <Path>/test-amp1.ngspice.tf   : Results file - transfer functn analysis  *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef PRCNGSPICE_HPP
#define PRCNGSPICE_HPP

// System Includes


// wxWindows Includes


// Application Includes

#include "PrcSimrBase.hpp"
#include "Component.hpp"
#include "ConvertType.hpp"
#include "PnlNgSpiceDC.hpp"
#include "PnlNgSpiceAC.hpp"
#include "PnlNgSpiceTR.hpp"

// Local Constant Declarations


//*****************************************************************************

class PrcNgSpice : public PrcSimrBase
{
  private:

    // Storage for the field list (used to replace Ng-Spice field identifiers) 
    wxString  m_osFieldLst;

    // Functions to parse simulator commands
    bool  bParseCmdOPT( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdOP ( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdIC ( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdDC ( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdAC ( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdTR ( wxString & rosCmd, Simulation & roSim );
    bool  bParseCmdPR ( wxString & rosCmd, Simulation & roSim );
    
    // Function to match a component to a set of node labels
    bool  bMatchCpnt( Simulation & roSim, wxString & rosToNodes );

    // Functions to create the simulator commands
    void  MakeCmdDC( Simulation & roSim );
    void  MakeCmdAC( Simulation & roSim );
    void  MakeCmdTR( Simulation & roSim );
    bool  bMakeCmds( Simulation & roSim );

    // Create arguments for the PRINT command
    wxString & rosMakeArgsPR( Simulation & roSim, wxString & rosPrefix );

    // Format the results file
    bool  bFmtResultsFile( void );
    bool  bFmtResultsHead( wxString & rosLine );
    bool  bFmtResultsLine( wxString & rosLine );

  public:

    PrcNgSpice( void );
    ~PrcNgSpice( );

    virtual  bool  bParseSim( Simulation & roSim );
    
    virtual  bool  bMakeSim( Simulation & roSim );

    virtual  bool  bExec( void );
};

//*****************************************************************************

#endif // PRCNGSPICE_HPP
